@@ -513,6 +513,7 @@ bool MySQL_Protocol::generate_pkt_EOF(bool send, void **ptr, unsigned int *len,
 		if ((*myds)->myconn->options.no_backslash_escapes) {
 			internal_status += SERVER_STATUS_NO_BACKSLASH_ESCAPES;
 		}
+		(*myds)->pkt_sid=sequence_id;
 	}
 	memcpy(_ptr+l, &warnings, sizeof(uint16_t)); l+=sizeof(uint16_t);
 	memcpy(_ptr+l, &internal_status, sizeof(uint16_t));
@@ -536,6 +537,9 @@ bool MySQL_Protocol::generate_pkt_EOF(bool send, void **ptr, unsigned int *len,
 #ifdef DEBUG
 	if (dump_pkt) { __dump_pkt(__func__,_ptr,size); }
 #endif
+	if (*myds) {
+		(*myds)->pkt_sid=sequence_id;
+	}
 	return true;
 }
 
@@ -585,6 +589,9 @@ bool MySQL_Protocol::generate_pkt_ERR(bool send, void **ptr, unsigned int *len,
 			if ((*myds)->sess)
 				if ((*myds)->sess->thread)
 					(*myds)->sess->thread->status_variables.generated_pkt_err++;
+	if (*myds) {
+		(*myds)->pkt_sid=sequence_id;
+	}
 	return true;
 }
 
@@ -653,6 +660,9 @@ bool MySQL_Protocol::generate_pkt_OK(bool send, void **ptr, unsigned int *len, u
 #ifdef DEBUG
 	if (dump_pkt) { __dump_pkt(__func__,_ptr,size); }
 #endif
+	if (*myds) {
+		(*myds)->pkt_sid=sequence_id;
+	}
 	return true;
 }
 
@@ -802,9 +812,7 @@ bool MySQL_Protocol::generate_pkt_field(bool send, void **ptr, unsigned int *len
 	memcpy(_ptr+l,&charset,sizeof(uint16_t)); l+=sizeof(uint16_t);
 	memcpy(_ptr+l,&column_length,sizeof(uint32_t)); l+=sizeof(uint32_t);
 	_ptr[l]=type; l++;
-	uint16_t flags_ = flags;
-	flags_ = flags_ & ~NUM_FLAG;
-	memcpy(_ptr+l,&flags_,sizeof(uint16_t)); l+=sizeof(uint16_t);
+	memcpy(_ptr+l,&flags,sizeof(uint16_t)); l+=sizeof(uint16_t);
 	_ptr[l]=decimals; l++;
 	_ptr[l]=0x00; l++;
 	_ptr[l]=0x00; l++;
@@ -2045,15 +2053,20 @@ stmt_execute_metadata_t * MySQL_Protocol::get_binds_from_pkt(void *ptr, unsigned
 						memcpy(binds[i].buffer,&ts,sizeof(MYSQL_TIME));
 					}
 					break;
+				case MYSQL_TYPE_DECIMAL:
+				case MYSQL_TYPE_VARCHAR:
+				case MYSQL_TYPE_BIT:
+				case MYSQL_TYPE_JSON:
+				case MYSQL_TYPE_NEWDECIMAL:
+				case MYSQL_TYPE_ENUM:
+				case MYSQL_TYPE_SET:
 				case MYSQL_TYPE_TINY_BLOB:
 				case MYSQL_TYPE_MEDIUM_BLOB:
 				case MYSQL_TYPE_LONG_BLOB:
 				case MYSQL_TYPE_BLOB:
-				case MYSQL_TYPE_VARCHAR:
 				case MYSQL_TYPE_VAR_STRING:
 				case MYSQL_TYPE_STRING:
-				case MYSQL_TYPE_DECIMAL:
-				case MYSQL_TYPE_NEWDECIMAL:
+				case MYSQL_TYPE_GEOMETRY:
 					{
 						uint8_t l=0;
 						uint64_t len;
@@ -2068,6 +2081,9 @@ stmt_execute_metadata_t * MySQL_Protocol::get_binds_from_pkt(void *ptr, unsigned
 					}
 					break;
 				default:
+					proxy_error("Unsupported field type %d in zero-based parameters[%d] "
+							"of query %s from user %s with default schema %s\n",
+							buffer_type, i, stmt_info->query, stmt_info->username, stmt_info->schemaname);
 					assert(0);
 					break;
 			}
@@ -2092,7 +2108,7 @@ stmt_execute_metadata_t * MySQL_Protocol::get_binds_from_pkt(void *ptr, unsigned
 
 MySQL_ResultSet::MySQL_ResultSet() {
 	buffer = NULL;
-	reset_pid = true;
+	//reset_pid = true;
 }
 void MySQL_ResultSet::init(MySQL_Protocol *_myprot, MYSQL_RES *_res, MYSQL *_my, MYSQL_STMT *_stmt) {
 	transfer_started=false;
@@ -2110,15 +2126,15 @@ void MySQL_ResultSet::init(MySQL_Protocol *_myprot, MYSQL_RES *_res, MYSQL *_my,
 	if (myprot) { // if myprot = NULL , this is a mirror
 		myds=myprot->get_myds();
 	}
-	if (reset_pid==true) {
+	//if (reset_pid==true) {
 	sid=0;
 	//PSarrayOUT = NULL;
 	if (myprot) { // if myprot = NULL , this is a mirror
 		sid=myds->pkt_sid+1;
 		//PSarrayOUT = new PtrSizeArray(8);
 	}
-	}
-	reset_pid=true;
+	//}
+	//reset_pid=true;
 	result=_res;
 	resultset_size=0;
 	num_rows=0;
