@@ -254,9 +254,20 @@ MySQL_Connection::~MySQL_Connection() {
 	}
 	if (mysql) {
 		// always decrease the counter
-		if (ret_mysql)
+		if (ret_mysql) {
 			__sync_fetch_and_sub(&MyHGM->status.server_connections_connected,1);
-		async_free_result();
+			if (query.stmt_result) {
+				if (query.stmt_result->handle) {
+					query.stmt_result->handle->status = MYSQL_STATUS_READY; // avoid calling mthd_my_skip_result()
+				}
+			}
+			if (mysql_result) {
+				if (mysql_result->handle) {
+					mysql_result->handle->status = MYSQL_STATUS_READY; // avoid calling mthd_my_skip_result()
+				}
+			}
+			async_free_result();
+		}
 		close_mysql(); // this take care of closing mysql connection
 		mysql=NULL;
 	}
@@ -1524,6 +1535,7 @@ int MySQL_Connection::async_set_autocommit(short event, bool ac) {
 		case ASYNC_SET_AUTOCOMMIT_FAILED:
 			return -1;
 			break;
+		case ASYNC_QUERY_END:
 		case ASYNC_IDLE:
 			set_autocommit(ac);
 			async_state_machine=ASYNC_SET_AUTOCOMMIT_START;
@@ -1851,7 +1863,7 @@ void MySQL_Connection::optimize() {
 // if avoids that a QUIT command stops forever
 // FIXME: currently doesn't support encryption and compression
 void MySQL_Connection::close_mysql() {
-	if ((send_quit) && (mysql->net.pvio)) {
+	if ((send_quit) && (mysql->net.pvio) && ret_mysql) {
 		char buff[5];
 		mysql_hdr myhdr;
 		myhdr.pkt_id=0;
@@ -1887,7 +1899,7 @@ int MySQL_Connection::async_send_simple_command(short event, char *stmt, unsigne
 	switch (async_state_machine) {
 		case ASYNC_QUERY_END:
 			processing_multi_statement=false;	// no matter if we are processing a multi statement or not, we reached the end
-			return 0;
+			//return 0; <= bug. Do not return here, because we need to reach the if (async_state_machine==ASYNC_QUERY_END) few lines below
 			break;
 		case ASYNC_IDLE:
 			set_query(stmt,length);
