@@ -24,7 +24,6 @@ struct bio_st {
 	CRYPTO_RWLOCK *lock;
 };
 
-
 #define RESULTSET_BUFLEN_DS_16K 16000
 #define RESULTSET_BUFLEN_DS_1M 1000*1024
 
@@ -217,6 +216,10 @@ MySQL_Data_Stream::MySQL_Data_Stream() {
 	CompPktOUT.partial=0;
 	multi_pkt.ptr=NULL;
 	multi_pkt.size=0;
+
+	statuses.questions = 0;
+	statuses.myconnpoll_get = 0;
+	statuses.myconnpoll_put = 0;
 }
 
 // Destructor
@@ -871,7 +874,7 @@ int MySQL_Data_Stream::buffer2array() {
 	}
 	if ((queueIN.pkt.size>0) && (queueIN.pkt.size==queueIN.partial) ) {
 		if (myconn->get_status_compression()==true) {
-			Bytef *dest;
+			Bytef *dest = NULL;
 			uLongf destLen;
 			proxy_debug(PROXY_DEBUG_PKT_ARRAY, 5, "Copied the whole compressed packet\n");
 			unsigned int progress=0;
